@@ -8,8 +8,9 @@ import re
 import sys
 from urllib import request, parse
 
-__version__ = "0.2.2"
+__version__ = "0.2.4"
 
+dry_run = False
 force = False
 
 fake_headers = {
@@ -350,6 +351,10 @@ class DummyProgressBar:
 
 def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None, merge = True, faker = False):
     assert urls
+    if dry_run:
+        print('Real URLs:\n', urls, '\n')
+        return
+    
     assert ext in ('3gp', 'flv', 'mp4', 'webm')
     if not total_size:
         try:
@@ -387,6 +392,7 @@ def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None,
             bar.update_piece(i + 1)
             url_save(url, filepath, bar, refer = refer, is_part = True, faker = faker)
         bar.done()
+        
         if not merge:
             print()
             return
@@ -396,10 +402,20 @@ def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None,
             for part in parts:
                 os.remove(part)
         elif ext == 'mp4':
-            from .processor.join_mp4 import concat_mp4
-            concat_mp4(parts, os.path.join(output_dir, title + '.mp4'))
-            for part in parts:
-                os.remove(part)
+            try:
+                from .processor.join_mp4 import concat_mp4
+                concat_mp4(parts, os.path.join(output_dir, title + '.mp4'))
+                for part in parts:
+                    os.remove(part)
+            except:
+                from .processor.ffmpeg import has_ffmpeg_installed
+                if has_ffmpeg_installed():
+                    from .processor.ffmpeg import ffmpeg_concat_mp4_to_mpg
+                    ffmpeg_concat_mp4_to_mpg(parts, os.path.join(output_dir, title + '.mp4'))
+                    for part in parts:
+                        os.remove(part)
+                else:
+                    print('No ffmpeg is found. Merging aborted.')
         else:
             print("Can't merge %s files" % ext)
     
@@ -407,13 +423,17 @@ def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None,
 
 def download_urls_chunked(urls, title, ext, total_size, output_dir = '.', refer = None, merge = True, faker = False):
     assert urls
+    if dry_run:
+        print('Real URLs:\n', urls, '\n')
+        return
+    
     assert ext in ('ts')
     title = escape_file_path(title)
-    filename = '%s.%s' % (title, ext)
+    filename = '%s.%s' % (title, 'ts')
     filepath = os.path.join(output_dir, filename)
     if total_size:
-        if not force and os.path.exists(filepath) and os.path.getsize(filepath) >= total_size * 0.9:
-            print('Skipping %s: file already exists' % tr(filepath))
+        if not force and os.path.exists(filepath[:-3] + '.mkv'):
+            print('Skipping %s: file already exists' % tr(filepath[:-3] + '.mkv'))
             print()
             return
         bar = SimpleProgressBar(total_size, len(urls))
@@ -421,10 +441,28 @@ def download_urls_chunked(urls, title, ext, total_size, output_dir = '.', refer
         bar = PiecesProgressBar(total_size, len(urls))
     
     if len(urls) == 1:
+        parts = []
         url = urls[0]
         print('Downloading %s ...' % tr(filename))
+        filepath = os.path.join(output_dir, filename)
+        parts.append(filepath)
         url_save_chunked(url, filepath, bar, refer = refer, faker = faker)
         bar.done()
+        
+        if not merge:
+            print()
+            return
+        if ext == 'ts':
+            from .processor.ffmpeg import has_ffmpeg_installed
+            if has_ffmpeg_installed():
+                from .processor.ffmpeg import ffmpeg_convert_ts_to_mkv
+                ffmpeg_convert_ts_to_mkv(parts, os.path.join(output_dir, title + '.mkv'))
+                for part in parts:
+                    os.remove(part)
+            else:
+                print('No ffmpeg is found. Conversion aborted.')
+        else:
+            print("Can't convert %s files" % ext)
     else:
         parts = []
         print('Downloading %s.%s ...' % (tr(title), ext))
@@ -436,14 +474,19 @@ def download_urls_chunked(urls, title, ext, total_size, output_dir = '.', refer
             bar.update_piece(i + 1)
             url_save_chunked(url, filepath, bar, refer = refer, is_part = True, faker = faker)
         bar.done()
+        
         if not merge:
             print()
             return
         if ext == 'ts':
-            from .processor.join_ts import concat_ts
-            concat_ts(parts, os.path.join(output_dir, title + '.ts'))
-            for part in parts:
-                os.remove(part)
+            from .processor.ffmpeg import has_ffmpeg_installed
+            if has_ffmpeg_installed():
+                from .processor.ffmpeg import ffmpeg_concat_ts_to_mkv
+                ffmpeg_concat_ts_to_mkv(parts, os.path.join(output_dir, title + '.mkv'))
+                for part in parts:
+                    os.remove(part)
+            else:
+                print('No ffmpeg is found. Merging aborted.')
         else:
             print("Can't merge %s files" % ext)
     
@@ -507,6 +550,18 @@ def set_http_proxy(proxy):
     opener = request.build_opener(proxy_support)
     request.install_opener(opener)
 
+def download_main(download, download_playlist, urls, playlist, output_dir, merge, info_only):
+    for url in urls:
+        if url.startswith('https://'):
+            url = url[8:]
+        if not url.startswith('http://'):
+            url = 'http://' + url
+        
+        if playlist:
+            download_playlist(url, output_dir = output_dir, merge = merge, info_only = info_only)
+        else:
+            download(url, output_dir = output_dir, merge = merge, info_only = info_only)
+
 def script_main(script_name, download, download_playlist = None):
     version = 'You-Get %s, a video downloader.' % __version__
     help = 'Usage: %s [OPTION]... [URL]...\n' % script_name
@@ -517,15 +572,16 @@ def script_main(script_name, download, download_playlist = None):
     help += '''\nDownload options (use with URLs):
     -f | --force                             Force overwriting existed files.
     -i | --info                              Display the information of videos without downloading.
-    -l | --playlist                          Download playlists. (only available for some sites)
+    -u | --url                               Display the real URLs of videos without downloading.
     -n | --no-merge                          Don't merge video parts.
     -o | --output-dir <PATH>                 Set the output directory for downloaded videos.
     -x | --http-proxy <PROXY-SERVER-IP:PORT> Use specific HTTP proxy for downloading.
          --no-proxy                          Don't use any proxy. (ignore $http_proxy)
+         --debug                             Show traceback on KeyboardInterrupt.
     '''
     
-    short_opts = 'Vhfino:x:'
-    opts = ['version', 'help', 'force', 'info', 'no-merge', 'no-proxy', 'output-dir=', 'http-proxy=']
+    short_opts = 'Vhfiuno:x:'
+    opts = ['version', 'help', 'force', 'info', 'url', 'no-merge', 'no-proxy', 'debug', 'output-dir=', 'http-proxy=']
     if download_playlist:
         short_opts = 'l' + short_opts
         opts = ['playlist'] + opts
@@ -542,6 +598,7 @@ def script_main(script_name, download, download_playlist = None):
     merge = True
     output_dir = '.'
     proxy = None
+    traceback = False
     for o, a in opts:
         if o in ('-V', '--version'):
             print(version)
@@ -555,12 +612,17 @@ def script_main(script_name, download, download_playlist = None):
             force = True
         elif o in ('-i', '--info'):
             info_only = True
+        elif o in ('-u', '--url'):
+            global dry_run
+            dry_run = True
         elif o in ('-l', '--playlist'):
             playlist = True
         elif o in ('-n', '--no-merge'):
             merge = False
         elif o in ('--no-proxy'):
             proxy = ''
+        elif o in ('--debug'):
+            traceback = True
         elif o in ('-o', '--output-dir'):
             output_dir = a
         elif o in ('-x', '--http-proxy'):
@@ -574,13 +636,10 @@ def script_main(script_name, download, download_playlist = None):
     
     set_http_proxy(proxy)
     
-    for url in args:
-        if url.startswith('https://'):
-            url = url[8:]
-        if not url.startswith('http://'):
-            url = 'http://' + url
-        
-        if playlist:
-            download_playlist(url, output_dir = output_dir, merge = merge, info_only = info_only)
-        else:
-            download(url, output_dir = output_dir, merge = merge, info_only = info_only)
+    if traceback:
+        download_main(download, download_playlist, args, playlist, output_dir, merge, info_only)
+    else:
+        try:
+            download_main(download, download_playlist, args, playlist, output_dir, merge, info_only)
+        except KeyboardInterrupt:
+            sys.exit(1)
